/******************************************************************************
*                                                                             *
*   TheTrap 0.1beta                                                           *
*                                                                             *
*   by Frank Adaemmer  copyrights[at]gmx.de                                   *
*                                                                             *
*   Source Code                                                               *
*                                                                             *
*   File:   install.c                                                         *
*   Use:    - source code file                                                *
*           - installation routine of the hook                                *
*                                                                             *
*******************************************************************************

TheTrap - A utility for TI's 68k calculaters

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
// C Source File
// Created 02/13/2001; 23:59:18 PM
//Original code from: Sample TSR by Greg Dietsche - Last Modification: 10/22/2002
//Modified by: Thomas Naeff, 16.03.2006
//Modified by: Frank Adaemmer, 03.03.2008

	//Set it to the same as in the Hook.c (doesn't care about the number of characters)
	#define TSR_NAME "TheTrap "

//no need for exit or atexit
  #define NO_EXIT_SUPPORT

//TODO: You Should Set MIN_AMS, and uncomment NO_AMS_CHECK
//If your code will not run on all available AMS versions
  #define MIN_AMS 207
//#define NO_AMS_CHECK

//Usually this results in smaller code, OPTIMIZE_ROM_CALLS also makes calling rom functions faster
//than they would be normally
//#define OPTIMIZE_ROM_CALLS

//TODO: Do you need to have TIGCC detect the calc type?
//For Sample TSR, NO_CALC_DETECT will result in smaller code
//You should comment this out to see if smaller code is produced in your own projects...
//Often times, undefining (commenting out) NO_CALC_DETECT will result in
//smaller code if you use the so-called psuedo constants such as KEY_UP in compat.h
	#define NO_CALC_DETECT

//TODO: Select the calculators that you are compiling for...
//In the project otions, maybe you should uncheck all checkmarks at
//Project -> Options... -> Compilation -> Calculator
//(If these project options are available in your version of TIGCC)
	#define USE_TI89
	#define USE_TI92PLUS
	#define USE_V200
	
	#define EVHK_TAG_HEXA (long)0x6576486B // = 'e' 'v' 'H' 'k'

	#define fatal(s) {ST_showHelp(s);return;}

	#include <tigcclib.h>			//Include All Header Files
	#include "h220xTSR.h"		 //Kevin Kofler's hw2 ams2 tsr support
	#include "array.h"			//The pre-compiled EVENT HOOK


//This Code needs TIGCC 0.94 beta 19 at the very least to function properly
//Thomas: I set this to TIGCC 0.96 beta 6 because I tested it with this version.
#if __TIGCC__==0 && __TIGCC_MINOR__< 96 && __TIGCC_BETA__ < 6
	#warning Sample TSR - Old TIGCC Version -- Check tigcc.ticalc.org for updates
#endif

// Main Function
void _main(void)
{
	volatile HANDLE tsrHandle; 		 //Handle of allocated mem for the Hook
	char		*pTsr;	 							//a pointer to the actual mem for the Hook
	void 		*pHook=EV_hook-16;	 //points to where evHk should be in an installed hook

/////////////////////////////////////////
// Program independent error checking	//
///////////////////////////////////////

	//see if the hook has already been installed
		if(EV_hook){
			char *HookPtr = (char*)EV_hook;
			char *PrevHookPtr = NULL; // points to the previous hook in the linked list
			while (HookPtr != NULL)
			{
				HookPtr -= 16; // point to the header of the hook
				
				// if the hook isn't following Kevin Kofler's convention,
				// our hook can't be further in the linked list
				if(!memcmp(HookPtr,tsr,12)){
					// if our hook is not the first in the linked list
					if (PrevHookPtr
						// Remove our hook from the linked list :
						// the next hook of PrevHookPtr becomes the next hook of HookPtr...
						// but only if the next hook follows Kevin's convention !
						&& (*(long*)(*(char**)(HookPtr+12) - 16) == EVHK_TAG_HEXA))
							*(char**)(PrevHookPtr+12) = *(char**)(HookPtr+12);
					else
						// Our hook is the first in the linked list :
						// EV_hook becomes the hook after our hook in the list
						EV_hook = (EVENT_HANDLER)*(char**)(HookPtr+12);
					// Free the block where the TSR was.
					// (don't forget to make HookPtr leave the ghost space)
					HeapFree (HeapPtrToHandle ((void*)((long)HookPtr ^ (HW_VERSION == 2 ? 0x40000 : 0))));
					fatal(TSR_NAME"deinstalled!");
				}

			//Get the address for the next hook...
				(unsigned long)pHook=*(unsigned long*)(pHook+12)-16;
			}
		}

//TODO: add your own code here... ie handle parameters, and do other stuff that should be
//done before your TSR Event Hook is installed

///////////////////////////////////
// End Error checking/handling	//
/////////////////////////////////

	//sort of like a call to a permanent version of enter_ghost_space();
		if(!h220xTSR())
			fatal("Low RAM!");

	//allocate RAM for the Event Hook
		if(!(tsrHandle=HeapAllocHigh(sizeof(tsr))))
			fatal("Low RAM!");

	//get a pointer to the actual mem and make it point in the "ghost space"
	//ghost space is essentially a shadow of regular ram which is not protected very
	//well by the hardware... subtract 0x40000 to get the real address
		pTsr=HeapDeref(tsrHandle)+0x40000;

	//Copy The tsr array to its new location
		memcpy(pTsr,tsr,sizeof(tsr));

	//Save the Old tsr hook's address for the uninstall prog and for this hook to call
		*(unsigned long*)&pTsr[12]=(unsigned long)EV_hook;

		EX_patch(pTsr,pTsr+sizeof(tsr));
		ASM_fastcall(&pTsr[16]);
		EV_hook=(EVENT_HANDLER)&pTsr[16];

	//TODO: You should change or remove this line as necessary
	//maybe you have to remove the mark in the poject options at that the messages will stay at the status line
	//Project -> Options... -> Compilation -> Programm Options -> Home Screen -> Save/Restore LCD contents
	//If you cleared the screen, you have to save and restore the LCD screen! (manually or by setting the mark)
		fatal(TSR_NAME"installed");
}




